#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Investing.com API - Market and historical data downloader
# https://github.com/crapher/pyinvesting.git
#
# Copyright 2020 Diego Degese
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from . import __user_agent__
from .online_scrapping import OnlineScrapping
from .online_websocket import OnlineWebsocket

class Online:
    
    def __init__(self, on_open=None, on_quotes=None, on_heartbeat=None, 
        on_error=None, on_close=None, proxy_url=None):
        """
        Class constructor 
        
        Parameters
        ----------
        on_open : function(self), optional
            Callable object which is called at opening websocket.
            This function has one argument. The argument is the callable object.
        on_quotes: function(self, quotes), optional
            Callable object which is called when received data.
            This function has 2 arguments.
                The 1st argument is the callable object.
                The 2nd argument is the dataframe with the quotes.
        on_heartbeat : function(self), optional
            Callable object which is called when a heartbeat response is received.
            This function has one argument. The argument is the callable object.
        on_error: function(self, exception), optional
            Callable object which is called when we get error.
            This function has 2 arguments.
                The 1st argument is the callable object.
                The 2nd argument is the exception object.
        on_close: function(self), optional
            Callable object which is called when closed the connection.
            This function has one argument. The argument is the callable object.
        proxy_url : str, optional
            The proxy URL with one of the following formats:
                - scheme://user:pass@hostname:port 
                - scheme://user:pass@ip:port
                - scheme://hostname:port 
                - scheme://ip:port
            
            Ex. https://john:doe@10.10.1.10:3128
        """
                
        self._pid_map = {}
        
        self._scrapping = OnlineScrapping(proxy_url=proxy_url)
        self._websocket = OnlineWebsocket(
            on_open = self._internal_on_open, 
            on_quotes = self._internal_on_quotes,
            on_heartbeat = self._internal_on_heartbeat, 
            on_error = self._internal_on_error, 
            on_close = self._internal_on_close,
            proxy_url = proxy_url)
        
        self._on_open = on_open
        self._on_quotes = on_quotes
        self._on_heartbeat = on_heartbeat
        self._on_error = on_error
        self._on_close = on_close
        
########################
#### PUBLIC METHODS ####
########################
    def connect(self):       
        """
        Connects to websocket to receive quotes information.
        """
        
        try:
            stream_server = self._scrapping.get_stream_server()
            self._websocket.connect(stream_server)
        except Exception as ex:
            self._internal_on_error(ex)
        
    def disconnect(self):
        """
        Disconnects from websocket to stop receiving quotes information.
        """
        
        try:
            self._websocket.disconnect()
        except Exception as ex:
            self._internal_on_error(ex)

    def subscribe(self, pair_id, ticker=None, link=None):
        """
        Subscribe to an asset to receive its quote information.
        
        Parameters
        ----------
        pair_id : int
            The pair_id that identify the asset to be retrieved.
        ticker : str, option
            The name of the ticker to be retrieved.
            If it is not specified, the ticker is the dataframe will be the pair_id
        link : str, optional
            The link received in the search ticket query. 
            If it is specified, it will be used to get the data, previous to subscribe it to the websocket connection.
        """
        
        self._pid_map[pair_id] = ticker if ticker else pair_id

        if link:
            try:
                quotes = self._scrapping.get_quotes_from_link(pair_id, link)
                self._internal_on_quotes(quotes)
            except:
                pass
                
        self._websocket.subscribe_event(pair_id)
        
########################
#### RTWS CALLBACKS ####        
########################
    def _internal_on_open(self):
        
        if self._on_open:
            self._on_open(self)
            
    def _internal_on_quotes(self, quotes):
                
        if self._on_quotes:
            column = quotes['pair_id'].apply(lambda x: self._pid_map[x] if x in self._pid_map else x)
            quotes.insert(0, 'ticker', column)
            quotes.set_index('pair_id', inplace=True)
            self._on_quotes(self, quotes)

    def _internal_on_heartbeat(self):

        if self._on_heartbeat:
            self._on_heartbeat(self)
        
    def _internal_on_error(self, error):

        if self._on_error:
            self._on_error(self, error)
        
    def _internal_on_close(self):

        if self._on_close:
            self._on_close(self)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Investing.com API - Market and historical data downloader
# https://github.com/crapher/pyinvesting.git
#
# Copyright 2020 Diego Degese
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from . import __user_agent__

from pyquery import PyQuery as pq

import re
import requests as rq
import pandas as pd
import numpy as np

import threading
import time
import json

class OnlineScrapping:
    
    def __init__(self, proxy_url=None):
        """
        Class constructor 
        
        Parameters
        ----------
        proxy_url : str, optional
            The proxy URL with one of the following formats:
                - scheme://user:pass@hostname:port 
                - scheme://user:pass@ip:port
                - scheme://hostname:port 
                - scheme://ip:port
            
            Ex. https://john:doe@10.10.1.10:3128
        """
        
        if proxy_url:
            self._proxies = {'http': proxy_url, 'https': proxy_url}
        else:
            self._proxies = None
        
        self._stream_server = None
        self._stream_server_lock = threading.Lock()
    
########################
#### PUBLIC METHODS ####
########################
    def get_stream_server(self):
        """
        Returns a stream server to be used for the websocket to retrive quotes.        
        """
        with self._stream_server_lock:
            if not self._stream_server:
                content = self._get_page_content('https://api.investing.com/api/editions/streamer')
                servers = json.loads(content)
                self._stream_server = servers['stream_servers'][0]
                    
        return self._stream_server
            
    def get_quotes_from_link(self, pair_id, link):
        """
        Parses and returns a dataframe with the quotes for the specified pair_id.
        
        Parameters
        ----------
        pair_id : int
            The pair_id value received in the search ticker query.
        link : str
            The link value received in the search ticker query.        
        """
        
        content = self._get_page_content(link)
        
        with self._stream_server_lock:
            if not self._stream_server:
                self._stream_server = self._get_stream_server_from_page(content)
            
        return self._get_quotes_from_page(content, pair_id)

#########################
#### PRIVATE METHODS ####
#########################
    def _get_page_content(self, url):
        
        if url.startswith('/'): # relative URL
            url = 'https://www.investing.com{}'.format(url)
                        
        headers = {
            'User-Agent': __user_agent__,
            'Accept-Encoding': 'gzip, deflate'
        }
                
        response = rq.get(url = url, headers = headers, proxies = self._proxies)
        response.raise_for_status()
        
        return response.text

    def _get_quotes_from_page(self, text, pair_id):
        
        data = {'pair_id': pair_id, 'bid': np.NAN, 'ask': np.NAN, 'last': np.NAN, 'high': np.NAN, 'low': np.NAN, 'pcp': np.NAN, 'turnover': np.NAN, 'pc': np.NAN, 'timestamp': int(time.time())}        
        keys = list(data)
        doc = pq(text.encode("ascii", "ignore"))
        
        data['bid'] = self._get_number_or_nan(doc, "div[data-test='bid-value']", index=0)
        data['ask'] = self._get_number_or_nan(doc, "div[data-test='bid-value']", index=1)
        data['last'] = self._get_number_or_nan(doc, "span[data-test='instrument-price-last']")
        data['low'] = self._get_number_or_nan(doc, "div[class*='trading-hours_value']", index=0)
        data['high'] = self._get_number_or_nan(doc, "div[class*='trading-hours_value']", index=1)
        data['pcp'] = self._get_number_or_nan(doc, "span[data-test='instrument-price-change-percent']")
        data['pc'] = self._get_number_or_nan(doc, "dd[data-test='prevClose']")

        result = pd.DataFrame(data, index=[0])
        result.columns = ['pair_id', 'bid', 'ask', 'last', 'high', 'low', 'change', 'turnover', 'previous_close', 'timestamp']
        result['datetime'] = pd.to_datetime(result['timestamp'], unit='s')
        result = result[['pair_id', 'bid', 'ask', 'last', 'high', 'low', 'change', 'turnover', 'previous_close', 'datetime']]
        
        return result

    def _get_number_or_nan(self, doc, query, index=0):
        
        try:
            value = doc(query).text()
            value = value.replace(',', '')
            value = re.findall(r"[-]?(?:\d*\.\d+|\d+)", value)[index]
            return float(value)
        except:
            return np.NAN


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Investing.com API - Market and historical data downloader
# https://github.com/crapher/pyinvesting.git

"""Investing.com API - Market and historical data downloader"""

from setuptools import setup, find_packages
from pyinvesting import __version__
import io
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with io.open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pyinvesting',
    version=__version__,
    description='Investing.com API - Market and historical data downloader',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/crapher/pyinvesting',
    author='Diego Degese',
    author_email='ddegese@gmail.com',
    license='Apache',
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Development Status :: 3 - Alpha',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Topic :: Office/Business :: Financial',
        'Topic :: Office/Business :: Financial :: Investment',
        'Topic :: Scientific/Engineering :: Interface Engine/Protocol Translator',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    platforms=['any'],
    keywords='pandas, investing, online, historical, downloader, finance',
    packages=find_packages(exclude=['contrib', 'docs', 'tests', 'examples']),
    install_requires=['pandas>=1.0.0', 'numpy>=1.18.1', 'requests>=2.21.0', 'websocket-client>=0.57.0', 'pyquery>=1.2']
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Investing.com API - Market and historical data downloader
# https://github.com/crapher/pyinvesting.git
#
# Copyright 2020 Diego Degese
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from . import __user_agent__

import requests as rq
import pandas as pd
import urllib
import json

class Search():
    
    def __init__(self, proxy_url=None):
        """
        Class constructor 
        
        Parameters
        ----------
        proxy_url : str, optional
            The proxy URL with one of the following formats:
                - scheme://user:pass@hostname:port 
                - scheme://user:pass@ip:port
                - scheme://hostname:port 
                - scheme://ip:port
            
            Ex. https://john:doe@10.10.1.10:3128
        """
        
        if proxy_url:
            self._proxies = {'http': proxy_url, 'https': proxy_url}
        else:
            self._proxies = None
        
########################
#### PUBLIC METHODS ####
########################
    def tickers(self, search_term, limit=30):
        """
        Returns all the tickers found under the search_term.
        
        Parameters
        ----------
        search_term : str
            Term used to filter the tickers
        limit : int
            Maximum results count that will be retrieved
        """
                
        tickers = self._internal_search(search_term.lower(), 'quotes', limit)
        if not tickers.empty:
            tickers.link = tickers.link.apply(lambda x: 'https://www.investing.com{}'.format(x) if x != None and x[0] == '/' else x)
            tickers = tickers[['pairId', 'link', 'symbol', 'exchange', 'name', 'type']]
            tickers.columns = ['pair_id', 'link', 'symbol', 'exchange', 'name', 'type']
            return tickers

        return pd.DataFrame()

    def news(self, search_term, limit=30):
        """
        Returns all the news found under the search_term.
        
        Parameters
        ----------
        search_term : str
            Term used to filter the news
        limit : int
            Maximum results count that will be retrieved
        """
        
        news = self._internal_search(search_term.lower(), 'news', limit)
        if not news.empty:
            news.link = news.link.apply(lambda x: 'https://www.investing.com{}'.format(x) if x != None and x[0] == '/' else x)
            news.dateTimestamp = pd.to_datetime(news.dateTimestamp, unit='s')
            news = news[['dateTimestamp', 'name', 'link', 'providerName']]
            news.columns = ['datetime', 'name', 'link', 'provider']
            return news

        return pd.DataFrame()

    def articles(self, search_term, limit=30):
        """
        Returns all the articles found under the search_term.
        
        Parameters
        ----------
        search_term : str
            Term used to filter the articles
        limit : int
            Maximum results count that will be retrieved
        """
        
        articles = self._internal_search(search_term.lower(), 'articles', limit)
        if not articles.empty:
            articles.link = articles.link.apply(lambda x: 'https://www.investing.com{}'.format(x) if x != None and x[0] == '/' else x)
            articles.dateTimestamp = pd.to_datetime(articles.dateTimestamp, unit='s')
            articles = articles[['dateTimestamp', 'name', 'link', 'authorName', 'isEditorPick']]
            articles.columns = ['datetime', 'name', 'link', 'author', 'is_editor_pick']
            return articles
            
        return pd.DataFrame()
        
#########################
#### PRIVATE METHODS ####
#########################
    def _internal_search(self, search_term, search_type, limit):
        
        headers = {
            'User-Agent': __user_agent__, 
            'X-Requested-With': 'XMLHttpRequest', 
            'Content-Type':'application/x-www-form-urlencoded'}
        
        url = 'https://www.investing.com/search/service/searchTopBar'    
        payload = urllib.parse.urlencode(
            {'search_text': search_term, 'type': search_type, 'limit': limit})
        
        response = rq.post(url=url, data=payload, headers=headers, proxies=self._proxies)
        response.raise_for_status()
        
        json = response.json()
        return pd.json_normalize(json[search_type])
